<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Ratctpagar;
use Teknisa\Model\Financeiro\Entities\Adiantapagar;
use Teknisa\Model\Financeiro\Entities\Devolucaopag;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Modules\ContaPagar\Factory\ParcelamentoTituloFactory;

/**
 * Class ParcelamentoTituloService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class ParcelamentoTituloService {

    const ID_AJ_BOL_COMP               = '4';
    const BOLETO_ALTERADO              = '3';
    const ID_CNAB_N_ENVIADO            = '00';
    const TAMANHO_NRLANBOLPAG          = 10;
    const IDBOLPAGALTER_DEFAULT        = '1';
    const IDFORMCADBPAG_DEFAULT        = '1';
    const STATUS_BOLETO_PENDENTE       = 'A';
    const PERMITE_ALTERACAO_BOLETO     = '2';
    const PREFIXO_NOVOCODIGO_BOLPAG    = 'BOLETPAGAR';
    const NAO_PERMITE_ALTERACAO_BOLETO = '1';

    protected $entityManager;
    protected $parcelamentoTituloFactory;
    protected $fluxoDeCaixaService;
    protected $newCode;

    /**
     * @param EntityManager $entityManager
     * @param ParcelamentoTituloFactory $parcelamentoTituloFactory
     */
    public function __construct(EntityManager $entityManager, ParcelamentoTituloFactory $parcelamentoTituloFactory, FluxoDeCaixaService $fluxoDeCaixaService, NewCode $newCode)
    {
        $this->entityManager = $entityManager;
        $this->parcelamentoTituloFactory = $parcelamentoTituloFactory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
        $this->newCode = $newCode;
    }

    /**
     * Gera Parcelamento Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param array $parcelas
     * @throws \Exception
     */
    public function geraParcelamentoTitulo(
        $nrorg, $cdfilial, $nrlanbolpag, $parcelas = array()
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::geraParcTitulo($nrorg, $cdfilial, $nrlanbolpag, $parcelas);
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Gera Parcelamento Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param array $parcelas
     * @throws \Exception
     */
    public function geraParcTitulo(
        $nrorg, $cdfilial, $nrlanbolpag, $parcelas = array()
    ){
        try {
            
            // Variáveis de Inicialização
            $incboletos    = 0;
            $valortotal    = 0;
            $valortotalbol = 0;
            $nrnovoboleto  = '';
            
            // Busca Boleto Original
            /** @var Boletpagar $boletooriginal */
            $boletooriginal = $this->parcelamentoTituloFactory->buscaBoletPagar(
                $nrorg, $cdfilial, $nrlanbolpag
            );

            // Número Boleto Original
            $nrorigbolpag = $boletooriginal->getNrorigbolpag()?:$nrlanbolpag;

            // Valor Boleto Original
            $vrantboleto = $boletooriginal->getVrfinalbpag();

            // Busca Associados ao de Origem
            /** @var Boletpagar $boletoorigem */
            $boletoorigem = $this->parcelamentoTituloFactory->buscaBoletPagarOriginal(
                $nrorg, $cdfilial, $nrlanbolpag
            );

            // Valor Total Boleto(s)
            if($boletoorigem) {
                /** @var Boletpagar $boleto */
                foreach ($boletoorigem as $boleto)
                    $valortotalbol += $boleto->getVrfinalbpag();
            }else $valortotalbol = $vrantboleto;

            // Soma Valor Total Parcelas
            for($i = 0;$i<count($parcelas);$i++)
                $valortotal += $parcelas[$i]["VRFINALBPAG"];

            // Busca Contas Boleto Parcelado
            $contasoriginais = $this->parcelamentoTituloFactory->buscaContaOrigParc($nrorg, $cdfilial, $nrlanbolpag);
            
            // Busca Rateio Conta Boleto Parcelado
            /** @var Ratctpagar $rateio */
            $rateiosoriginais = $this->parcelamentoTituloFactory->buscaRatctpagar(
                $contasoriginais[0]['NRORG'], $contasoriginais[0]['CDFILIAL'], $contasoriginais[0]['NRLANCTONF'], $contasoriginais[0]['NRPARCELAPAG'], $contasoriginais[0]['NRSEQVENPAG']
            );

            // Percorre Parcelas
            for($i = 0;$i<count($parcelas);$i++){

                // Formata Data Vencimento Boleto
                $dtvenbolpag = self::formataData($parcelas[$i]['DTVENBOLPAG']);

                // Calcula Multa/Mora
                if($boletooriginal->getVrpemulbpag() > 0)
                    $vrmultabpag  = ($boletooriginal->getVrpemulbpag() * $parcelas[$i]['VRFINALBPAG'])/100;
                else  $vrmultabpag = null;
                if($boletooriginal->getVrpemorbpag() > 0)
                     $vrmordiabpag = ($boletooriginal->getVrpemorbpag() * $parcelas[$i]['VRFINALBPAG'])/100;
                else $vrmordiabpag = null;
                
                // Verifica Boleto Original
                if($i == 0){
                    
                    $nrparcbolpag = str_pad($parcelas[$i]["NRPARCBOLPAG"], 2, 0, STR_PAD_LEFT);
                    $nrparcbolpagvisu = $parcelas[$i]["NRPARCBOLPAGVISU"];
                    
                    // Atualiza Boleto Original
                    $boletooriginal = $this->parcelamentoTituloFactory->atualizaBoletpagarOriginal(
                        $boletooriginal, $parcelas[$i]['VRFINALBPAG'], $dtvenbolpag, null, null, null,
                        self::ID_CNAB_N_ENVIADO, null, round($parcelas[$i]['VRFINALBPAG'], 2),
                        self::STATUS_BOLETO_PENDENTE, $parcelas[$i]['CDMOTIPARC'], null, null,
                            $nrparcbolpag, $nrparcbolpagvisu, $nrorigbolpag,
                        $vrmordiabpag, $vrmultabpag
                    );
                    $this->entityManager->persist($boletooriginal);
                }else{
                    $nrparcbolpag = str_pad($parcelas[$i]["NRPARCBOLPAG"], 2, 0, STR_PAD_LEFT);
                    $nrparcbolpagvisu = $parcelas[$i]["NRPARCBOLPAGVISU"];
                    
                    // Incremento
                    $incboletos++;

                    // Busca Boleto Pagar
                    /** @var Boletpagar $boletonovo */
                    $boletonovo = $this->parcelamentoTituloFactory->buscaBoletPagar(
                        $parcelas[$i]["NRORG"], $parcelas[$i]["CDFILIAL"], $parcelas[$i]["NRLANBOLPAG"]
                    );

                    // Verifica Existência Boleto
                    if(!$boletonovo instanceof Boletpagar){
                        // Gera Novo Código Boleto
                        $novocodigo = $this->newCode->getCodeByTableName(self::PREFIXO_NOVOCODIGO_BOLPAG.$parcelas[$i]["CDFILIAL"], $nrorg);

                        // Insere Novo Boleto
                        /** @var Boletpagar $boletonovo */
                        $boletonovo = $this->parcelamentoTituloFactory->insereBoletpagar(
                            $nrorg, $boletooriginal->getCdfilial(), $novocodigo
                        );

                        // Novo Código Boleto
                        $nrnovoboleto = $novocodigo;
                    }else $nrnovoboleto = $parcelas[$i]["NRLANBOLPAG"];

                    // Atualiza Boleto
                    $boletonovo = $this->parcelamentoTituloFactory->atualizaBoletpagarNovo(
                        $boletonovo, $dtvenbolpag, $nrorigbolpag, $boletooriginal->getCdforneced(), $boletooriginal->getCdempresa(), $boletooriginal->getCdreducont(),
                        $boletooriginal->getVrpemulbpag(), $vrmultabpag, $boletooriginal->getVrpemorbpag(), $vrmordiabpag, $boletooriginal->getCdtpbaixapag(), self::STATUS_BOLETO_PENDENTE, self::ID_CNAB_N_ENVIADO,
                        $parcelas[$i]['VRFINALBPAG'], $parcelas[$i]['CDMOTIPARC'], $parcelas[$i]['VRFINALBPAG'], $nrparcbolpag, $nrparcbolpagvisu, null, null, null,
                        null, null, $boletooriginal->getCdoperinclusao()
                    );

                    $this->entityManager->persist($boletonovo);
                    $this->entityManager->flush();
                }
                
                // Variáveis
                $vrtotconta     = 0;
                $contadorcontas = 0;

                // Percorre Contas Boleto Parcelado
                foreach ($contasoriginais as $conta){
                    // Contador
                    $contadorcontas++;

                    // Busca Maxima Conta Parcelamento
                    $sqlmaxconta = $this->parcelamentoTituloFactory->buscaMaxCntParc(
                        $conta['NRORG'],
                        $conta['CDFILIAL'],
                        $conta['NRLANCTONF'],
                        $conta['NRSEQVENPAG']
                    );

                    // Variáveis
                    $maiornrconta = $sqlmaxconta[0]['MAXNRPARCPAG'];
                    $nrparcelapagn  = $maiornrconta;
                    $filialconta  = $conta['CDFILIAL'];
                    $nrlanctonf   = $conta['NRLANCTONF'];
                    $nrseqvenpag  = $conta['NRSEQVENPAG'];
                    $vrconta      = round($parcelas[$i]['VRFINALBPAG'] * $conta['VRATUAPAG']/ $vrantboleto, 2);
                    $vrtotconta  += $vrconta;
                    
                    if(round($valortotal, 2) != round($valortotalbol,2)){
                        /** @var Notafiscal $notafiscal */
                        $notafiscal = $this->parcelamentoTituloFactory->buscaNotafiscal(
                            $conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANCTONF']
                        );

                        $notafiscal->setVrprodutos($valortotal);
                        $notafiscal->setVrnotafisc($valortotal);
                        $this->entityManager->persist($notafiscal);
                    }

                    // Atualiza Fluxo de Caixa
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                        $nrorg, $conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'],
                        $parcelas[$i]['DTVENBOLPAG'], $vrconta, $conta['IDMOVFLUXO'], 'S'
                    );

                    $this->entityManager->persist($movfluxo);
                    
                    
                    $this->entityManager->flush();
                    // Verifica Boleto Original
                    if($i==0){
                        
                        $nrparcelapagn = ($conta['NRPARCELAPAG'] != '00') ? $conta['NRPARCELAPAG'] : $nrparcelapagn;
                        
                        $nrparcelapag = str_pad($nrparcelapagn, 2, 0, STR_PAD_LEFT);
                        $nrparcelapagvisu = str_pad($nrparcelapagn, 3, 0, STR_PAD_LEFT);
                        
                        
                        // Busca Rateio Conta Boleto Parcelado
                        /** @var Ratctpagar $rateio */
                        $rateios = $this->parcelamentoTituloFactory->buscaRatctpagar(
                            $nrorg, $filialconta, $nrlanctonf, $conta['NRPARCELAPAG'], $conta['NRSEQVENPAG']
                        );
                        
                        // Busca Rateio Conta Boleto Parcelado
                        /** @var $delrateio Ratctpagar */
                            $delrateio = $this->parcelamentoTituloFactory->buscaRatctpagar(
                            $nrorg, $filialconta, $nrlanctonf, $conta['NRPARCELAPAG'], $conta['NRSEQVENPAG']
                        );
                        
                        // Deleta Rateio Contas Boleto Parcelado
                        if((bool)$delrateio === true){
                            /** @var $rat Ratctpagar*/
                            foreach ($delrateio as $rat)
                                $this->entityManager->remove($rat);
                        }
                        $this->entityManager->flush();

                        // Atualiza Fluxo de Caixa
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'],
                            $conta['DTMOVFLUXO'], $conta['VRATUAPAG'], $conta['IDMOVFLUXO'], 'S'
                        );

                        $this->entityManager->persist($movfluxo);

                        /** @var $contapagar Contapagar */
                        $contapagar = $this->parcelamentoTituloFactory->buscaContapagar(
                            $nrorg, $filialconta, $nrlanctonf, $conta['NRPARCELAPAG'], $nrseqvenpag
                        );

                        // Atualiza Conta
                        $contapagar = $this->parcelamentoTituloFactory->atualizaContapagar(
                            $contapagar, $vrconta, round($vrconta - $conta['VRAJUSTE'], 2), $dtvenbolpag,
                            'E', $parcelas[$i]['CDMOTIPARC'], $nrparcelapag, $nrparcelapagvisu
                        );

                        // Verifica Diferença Soma Valor Total
                        if($contadorcontas == count($contasoriginais)){
                            if ((float) $vrtotconta != (float)$parcelas[$i]['VRFINALBPAG'])
                                $vrconta +=  $parcelas[$i]['VRFINALBPAG'] - $vrtotconta;
                        }
                        
                        $contapagar->setVratuapag($vrconta);


                        $this->entityManager->persist($contapagar);
                    }else{
                        /** @var $contapagar Contapagar */
                        $novaconta = $this->parcelamentoTituloFactory->buscaContapagarBoleto(
                            $parcelas[$i]['NRORG'], $parcelas[$i]['CDFILIAL'], $parcelas[$i]['NRLANBOLPAG']
                        );

                        if(!$novaconta instanceof Contapagar) {
                            
                            $nrparcelapagn  = $maiornrconta+1;
                            if($nrparcelapagn > 99)
                                $nrparcelapagAlfa =  $this->parcelamentoTituloFactory->buscaNrpacerlapagAlfa($nrparcelapagn);
                            else $nrparcelapagAlfa = $nrparcelapagn;
                            
                            // Gera Nº Parcela
                            $nrparcelapag = str_pad($nrparcelapagAlfa, 2, 0, STR_PAD_LEFT);//str_pad($maiornrconta+1,2,'0',STR_PAD_LEFT);
                            $nrparcelapagvisu = $nrparcelapagn;//str_pad($maiornrconta+1,2,'0',STR_PAD_LEFT);
                            
                            $dscontapag_parc = isset($parcelas[$i]['DSCONTAPAG'])?$parcelas[$i]['DSCONTAPAG']:$conta['DSCONTAPAG'];

                            // Insere Conta a Pagar
                            /** @var Contapagar $novaconta */
                            $novaconta = $this->parcelamentoTituloFactory->insereContapagar(
                                $filialconta, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $nrparcelapagvisu, $cdfilial, $nrnovoboleto, $vrconta, ($parcelas[$i]['CDMOTIPARC']),
                                $vrconta, $dtvenbolpag, $conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'], $dtvenbolpag, 'E', $conta['QTDIASVENPAG'],
                                $conta['IDSTATPAG'], $conta['CDTPCTPAGAR'], $dtvenbolpag, $dscontapag_parc, $nrorg
                            );
                            
                            $this->entityManager->flush();
                             
                            // Atualiza Fluxo de Caixa
                            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                $nrorg, $conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'],
                                $dtvenbolpag, $vrconta, 'E', 'S'
                            );
                            
                            $this->entityManager->persist($movfluxo);
                        }else{
                            // Busca Rateio Conta Boleto Parcelado
                            /** @var Ratctpagar $rateio */
                            $rateios = $this->parcelamentoTituloFactory->buscaRatctpagar(
                                $nrorg, $filialconta, $nrlanctonf, $conta['NRPARCELAPAG'], $conta['NRSEQVENPAG']
                            );
                            
                            // Busca Rateio Conta Boleto Parcelado
                            /** @var $delrateio Ratctpagar */
                                $delrateio = $this->parcelamentoTituloFactory->buscaRatctpagar(
                                $nrorg, $filialconta, $nrlanctonf, $novaconta->getNrparcelapag(), $novaconta->getNrseqvenpag()
                            );
                            
                            // Deleta Rateio Contas Boleto Parcelado
                            if((bool)$delrateio === true){
                                /** @var $rat Ratctpagar*/
                                foreach ($delrateio as $rat)
                                    $this->entityManager->remove($rat);
                            }
                            $this->entityManager->flush();
                        }
                        
                        $this->entityManager->flush();

                        // Verifica Diferença Soma Valor Total
                        if($contadorcontas == count($contasoriginais)){
                            if ((float) $vrtotconta != (float)$parcelas[$i]['VRFINALBPAG'])
                                $vrconta +=  $parcelas[$i]['VRFINALBPAG'] - $vrtotconta;
                        }

                        $novaconta->setDtatuavenpag(self::formataData($parcelas[$i]['DTVENBOLPAG']));
                        $novaconta->setDtorigvenpag(self::formataData($parcelas[$i]['DTVENBOLPAG']));
                        $novaconta->setVratuapag($vrconta);
                        $novaconta->setVrorigpag($vrconta);

                        // Gera Rateio Conta Pagar

                        $this->entityManager->persist($novaconta);
                    }
                    
                    $this->entityManager->flush();
                    $totalrateio = 0;
                    $contadorrateio = 0;
                    
                    // Percorre Rateios Conta Boleto Parcelado
                    /** @var Ratctpagar $rateio */
                    foreach ($rateiosoriginais as $rateio){
                        // Contador
                        $contadorrateio++;
                        
                        // Calcula Valor Rateio
                        $aux = $vrconta * $rateio->getVrratccctpag();
                        $vrrateio = $aux / $conta['VRATUAPAG'];
                        $totalrateio += $vrrateio;
                        
                        // Verifica Diferença Soma Valor Total
                        if($contadorrateio == count($rateios)){
                            if((float)$vrtotconta != (float)$vrconta)
                                $vrrateio = $vrrateio-($vrconta-$totalrateio);
                        }
                        // Insere Rateio
                        /** @var Ratctpagar $novorateio */
                        $novorateio = $this->parcelamentoTituloFactory->insereRatctpagar(
                            $nrorg, $filialconta, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateio->getCdcentcust(),
                            $rateio->getCdfiratctpag(), $vrrateio
                        );
                        
                        $this->entityManager->persist($novorateio);      
                        $this->entityManager->flush();
                    }
                }
            }
            
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Parcelamento Título
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return null|Boletpagar
     * @throws \Exception
     */
    public function validaParcelamentoTitulo(
        $nrorg, $cdfilial, $nrlanbolpag
    ){
        try {

            /** @var BoletPagar $boletpagar */
            $boletpagar = $this->parcelamentoTituloFactory->buscaBoletpagar(
                $nrorg,
                $cdfilial,
                $nrlanbolpag
            );

            if ($boletpagar->getIdstatbpag() == 'B')
                throw new \Exception('Operação bloqueada. Existe(m) parcela(s) baixada(s).');
            if ($boletpagar->getIdstatbpag() == 'E')
                throw new \Exception('Operação bloqueada. Existe(m) parcela(s) emitida(s).');
            if ($boletpagar->getIdsitucnabp() != '00')
                throw new \Exception('Operação bloqueada. Existe(m) parcela(s) vinculada(s) a remessa(s).');

            return $boletpagar;

        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }


    /**
     * Deleta Parcelamento Título
     * @param Boletpagar $boletoant
     * @param Boletpagar $boletoatual
     * @param $qtdparcelas
     * @throws \Exception
     */
    public function deletaParcelamentoTitulo(
        Boletpagar $boletoant, Boletpagar $boletoatual, $qtdparcelas
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            $vrtotbolpag = (round($boletoant->getVrbolpag(), 2) + round($boletoatual->getVrbolpag(), 2));

            //Updata o a parcela anterior com o valor do boleto a ser excluido
            $boletoant->setVrbolpag($vrtotbolpag);
            $boletoant->setVrfinalbpag($vrtotbolpag);
            $boletoant->setDslinhbarpag(null);
            $boletoant->setCdnosnumpag(null);
            $boletoant->setCdbarrapag(null);

            // Calcula Multa/Mora
            if($boletoant->getVrpemulbpag() > 0)
                $vrmultabpag  = ($boletoant->getVrpemulbpag() * $vrtotbolpag)/100;
            else  $vrmultabpag = null;
            if($boletoant->getVrpemorbpag() > 0)
                $vrmordiabpag = ($boletoant->getVrpemorbpag() * $vrtotbolpag)/100;
            else $vrmordiabpag = null;

            $boletoant->setVrmultabpag($vrmultabpag);
            $boletoant->setVrmordiabpag($vrmordiabpag);

            if($qtdparcelas == 2){
                $boletoant->setCdmotiparc('');
                $boletoant->setNrparcbolpag('00');
                $boletoant->setNrparcbolpagvisu('000');
            }

            $this->entityManager->persist($boletoant);

            // Busca os ajustes da parcela a ser excluÃ­da
            /** @var $ajustes Ajbolpag */
            $ajustes = $this->parcelamentoTituloFactory->buscaAjbolpags(
                $boletoatual->getNrorg(),
                $boletoatual->getCdfilial(),
                $boletoatual->getNrlanbolpag()
            );

            /** @var $ajuste Ajbolpag */
            foreach ($ajustes as $ajuste){

                /** @var $ajustebolant Ajbolpag */
                $ajustebolant = $this->parcelamentoTituloFactory->buscaAjbolpag(
                    $boletoant->getNrorg(),
                    $boletoant->getCdfilial(),
                    $boletoant->getNrlanbolpag(),
                    $ajuste->getCdajuctpag()
                );

                //se ja existir no boleto anteior o cÃ³digo de ajuste do boleto excluido updata o mesmo
                if((bool) $ajustebolant === true){
                    $ajustebolant->setVrajbolpag(round($ajustebolant->getVrajbolpag(), 2)+round($ajuste->getVrajbolpag(), 2));
                    $this->entityManager->persist($ajustebolant);
                }else{
                    //se nÃ£o existir insere
                    /** @var Ajbolpag $novoajuste */
                    $novoajuste = $this->parcelamentoTituloFactory->insereAjbolpag(
                        $ajuste->getNrorg(), $boletoant->getCdfilial(), $boletoant->getNrlanbolpag(),
                        $ajuste->getCdajuctpag(), (round($ajuste->getVrajbolpag(), 2))
                    );

                    $this->entityManager->persist($novoajuste);
                }

                //Busca existÃªncia de adiantamento para a parcela ant.
                /** @var $adiantamentoAnt Adiantapagar */
                $adiantamentoAnt = $this->parcelamentoTituloFactory->buscaAdiantapagar(
                    $boletoant->getNrorg(),
                    $boletoant->getCdfilial(),
                    $boletoant->getNrlanbolpag(),
                    $ajuste->getCdajuctpag()
                );

                //Busca existÃªncia de devolucao para a parcela ant.
                /** @var $devolucaoAnt Devolucaopag */
                $devolucaoAnt = $this->parcelamentoTituloFactory->buscaDevolucaopag(
                    $boletoant->getNrorg(),
                    $boletoant->getCdfilial(),
                    $boletoant->getNrlanbolpag(),
                    $ajuste->getCdajuctpag()
                );

                //Busca existÃªncia de adiantamento para a parcela atual.
                /** @var $adiantamentoAtual Adiantapagar */
                $adiantamentoAtual = $this->parcelamentoTituloFactory->buscaAdiantapagar(
                    $ajuste->getNrorg(),
                    $ajuste->getCdfilial(),
                    $ajuste->getNrlanbolpag(),
                    $ajuste->getCdajuctpag()
                );

                //Busca existÃªncia de devolucao para a parcela atual.
                /** @var $devolucaoAtual Devolucaopag */
                $devolucaoAtual = $this->parcelamentoTituloFactory->buscaDevolucaopag(
                    $ajuste->getNrorg(),
                    $ajuste->getCdfilial(),
                    $ajuste->getNrlanbolpag(),
                    $ajuste->getCdajuctpag()
                );

                $this->entityManager->remove($ajuste);
                $this->entityManager->flush();


                if ((bool) $adiantamentoAtual === true){

                    if((bool)$adiantamentoAnt === true){
                        $adiantamentoAnt->setVradiantapag(round($adiantamentoAnt->getVradiantapag(), 2)+round($adiantamentoAtual->getVradiantapag(), 2));
                        $this->entityManager->persist($adiantamentoAnt);
                    }else{
                        /** @var Adiantapagar $novoadiantamento */
                        $novoadiantamento = $this->parcelamentoTituloFactory->insereAdiantapagar(
                            $adiantamentoAtual->getNrorg(), $adiantamentoAtual->getCdfilial(), $adiantamentoAtual->getNrlanctonf(),
                            $adiantamentoAtual->getNrseqvenpag(), $adiantamentoAtual->getNrparcelapag(), $boletoant->getCdfilial(),
                            $boletoant->getNrlanbolpag(), $adiantamentoAtual->getCdajuctpag()
                        );

                        $this->entityManager->persist($novoadiantamento);
                    }
                }
                if((bool)$devolucaoAtual === true){

                    /** @var Notafiscal $notafiscal */
                    $notafiscal = $this->parcelamentoTituloFactory->buscaNotafiscal(
                        $devolucaoAtual->getNrorg(),
                        $devolucaoAtual->getCdfilial(),
                        $devolucaoAtual->getNrlanctonf()
                    );

                    if((bool) $devolucaoAnt === true){

                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl()+$devolucaoAtual->getVrdevolucaopag());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());

                        $this->entityManager->persist($notafiscal);

                        $devolucaoAnt->setVrdevolucaopag(round($devolucaoAnt->getVrdevolucaopag() + $devolucaoAtual->getVrdevolucaopag(), 2));

                        $this->entityManager->persist($devolucaoAnt);
                    }else{
                        $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl()+$devolucaoAtual->getVrdevolucaopag());
                        $notafiscal->setCdfilial($devolucaoAtual->getCdfilial());
                        $notafiscal->setNrlanctonf($devolucaoAtual->getNrlanctonf());

                        $this->entityManager->persist($notafiscal);

                        /** @var Devolucaopag $novadevolucao */
                        $novadevolucao = $this->parcelamentoTituloFactory->insereDevolucaopag(
                            $boletoant->getNrorg(), $devolucaoAtual->getCdfilial(), $devolucaoAtual->getNrlanctonf(),
                            $boletoant->getNrlanbolpag(), $devolucaoAtual->getCdajuctpag(),
                            $devolucaoAtual->getVrdevolucaopag()
                        );

                        $this->entityManager->persist($novadevolucao);
                    }

                }
            }

            //Busca conta do boleto excluido
            /** @var $contaspagar Contapagar */
            $contaspagar = $this->parcelamentoTituloFactory->buscaContaOrigParc(
                $boletoatual->getNrorg(), $boletoatual->getCdfilial(), $boletoatual->getNrlanbolpag()
            );

            $primconta = true;
            $countconta = 0;
            $contasoriginais = null;

            /** @var $conta Contapagar */
            foreach ($contaspagar as $conta){
                if ($primconta){
                    $contasoriginais = $this->parcelamentoTituloFactory->buscaContaOrigParc(
                        $boletoant->getNrorg(), $boletoant->getCdfilial(), $boletoant->getNrlanbolpag()
                    );
                    $primconta = false;
                }

                $ajustesconta = $this->parcelamentoTituloFactory->buscaAjusteConta(
                    $conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANBOLPAG']
                );

                foreach($ajustesconta as $ajusteconta){

                    // Busca Ajuste Conta Pagar
                    /** @var Ajctpagar $ajctpagar */
                    $ajctpagar = $this->parcelamentoTituloFactory->buscaAjctpagar(
                        $ajusteconta['NRORG'], $ajusteconta['CDFILIAL'], $ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENPAG'], $contasoriginais[$countconta]['NRPARCELAPAG'],
                        $ajusteconta['CDAJUCTPAG'], $ajusteconta['DTAJUSTEPAG']
                    );

                    // Verifica Existência Ajuste
                    if(!$ajctpagar){

                        // Insere Ajuste Conta Pagar
                        /** @var Ajctpagar $novoajuste */
                        $novoajuste = $this->parcelamentoTituloFactory->insereAjctpagar(
                            $ajusteconta['NRORG'], $ajusteconta['CDFILIAL'], $ajusteconta['NRLANCTONF'], $ajusteconta['DTAJUSTEPAG'],
                            $ajusteconta['CDAJUCTPAG'], $ajusteconta['DSAJUSTEPAG'],$ajusteconta['NRSEQVENPAG'],
                            $contasoriginais[$countconta]['NRPARCELAPAG'], $ajusteconta['CDCLASSFINA'],
                            $ajusteconta['CDEMMFAJCNTR'], $ajusteconta['CDFIMFAJCNTR'], $ajusteconta['DTMOVFLUXO'],
                            $ajusteconta['IDMOVFLUXO'], $ajusteconta['VRAJUSTEPAG']
                        );

                        $this->entityManager->persist($novoajuste);
                    }

                    // Busca Ajuste Conta Pagar
                    /** @var Ajctpagar $delajctpagar */
                    $delajctpagar = $this->parcelamentoTituloFactory->buscaAjctpagar(
                        $ajusteconta['NRORG'],$ajusteconta['CDFILIAL'],$ajusteconta['NRLANCTONF'],
                        $ajusteconta['NRSEQVENPAG'],$ajusteconta['NRPARCELAPAG'],
                        $ajusteconta['CDAJUCTPAG'],$ajusteconta['DTAJUSTEPAG']
                    );

                    $this->entityManager->remove($delajctpagar);
                }

                // Busca Conta Pagar Original
                /** @var $contaoriginal Contapagar*/
                $contaoriginal = $this->parcelamentoTituloFactory->buscaContapagar(
                    $contasoriginais[$countconta]['NRORG'], $contasoriginais[$countconta]['CDFILIAL'],
                    $contasoriginais[$countconta]['NRLANCTONF'], $contasoriginais[$countconta]['NRPARCELAPAG'],
                    $contasoriginais[$countconta]['NRSEQVENPAG']
                );

                $valorContaOld = round($contasoriginais[$countconta]['VRATUAPAG'], 2);

                $contaoriginal->setVratuapag(round($contasoriginais[$countconta]['VRATUAPAG'], 2)+round($conta['VRATUAPAG'], 2));
                $contaoriginal->setVrorigpag(round($contasoriginais[$countconta]['VRORIGPAG'], 2)+round($conta['VRORIGPAG'], 2));

                $this->entityManager->persist($contaoriginal);

                // Gera Novo Rateio Conta Pagar
                self::geraNovoRateioConta(
                    $contasoriginais[$countconta]['CDFILIAL'], $contasoriginais[$countconta]['NRLANCTONF'],
                    $contasoriginais[$countconta]['NRSEQVENPAG'],$contasoriginais[$countconta]['NRPARCELAPAG'],
                    round($contasoriginais[$countconta]['VRATUAPAG'], 2) + round($conta['VRATUAPAG'], 2),
                    $valorContaOld,$contasoriginais[$countconta]['NRORG']
                );

                // Busca Conta Pagar
                /** @var Contapagar $delconta */
                $delconta = $this->parcelamentoTituloFactory->buscaContapagar(
                    $conta['NRORG'], $conta['CDFILIAL'],
                    $conta['NRLANCTONF'], $conta['NRPARCELAPAG'],
                    $conta['NRSEQVENPAG']
                );

                // Deleta Conta Pagar
                $this->entityManager->remove($delconta);
            }

            $this->entityManager->flush();

            // Exclui Boleto Atual
            $this->entityManager->remove($boletoatual);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    /**
     * Gera Novo Rateio Conta Pagar
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $valornovo
     * @param $valorantigo
     * @param $nrorg
     * @throws \Exception
     */
    public function geraNovoRateioConta(
        $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $valornovo, $valorantigo, $nrorg
    ){
        try {

            // Busca Valore(s) Rateio(s)
            $rateios = $this->parcelamentoTituloFactory->buscaValorRateioConta(
                $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $nrorg
            );

            // Busca Rateio(s)
            /** @var Ratctpagar $delrateio */
            $delrateio = $this->parcelamentoTituloFactory->buscaRatctpagar(
                $nrorg, $cdfilial, $nrlanctonf, $nrparcelapag, $nrseqvenpag
            );

            // Deleta Rateio(s)
            if ((bool) $delrateio === true){
                /** @var $rat Ratctpagar*/
                foreach ($delrateio as $rat)
                    $this->entityManager->remove($rat);
            }

            $this->entityManager->flush();

            $totalrateio = 0;
            $contadorrateio = 0;

            /** @var $rateio Ratctpagar */
            foreach ($rateios as $rateio) {

                // Contador Rateio
                $contadorrateio++;

                // Calcula Valor Rateio
                $valorrateio  = round($valornovo * ($rateio['VRRATCCCTPAG'] / $valorantigo), 2);

                // Soma Valor Total Rateio
                $totalrateio += $valorrateio;

                // Insere Rateio Conta Pagar
                /** @var Ratctpagar $novorateio */
                $novorateio = $this->parcelamentoTituloFactory->insereRatctpagar(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateio['CDCENTCUST'], $rateio['CDFIRATCTPAG'], $valorrateio
                );

                // Verifica Diferença Valor[Rateio x Conta]
                if ($contadorrateio == count($rateios))
                    if ((float)$totalrateio != (float)$valornovo)
                        $novorateio = $this->parcelamentoTituloFactory->atualizaRatctpagarValor($novorateio, ($valorrateio - ($valornovo - $totalrateio)));

                $this->entityManager->persist($novorateio);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}